@@ -1,21 +1,22 @@
 package Net::IPAM::Tree;
 
+our $VERSION = '1.08';
+
 use 5.10.0;
 use strict;
 use warnings;
 use utf8;
 
-our $VERSION = '1.07';
-
-# caller can shut up with: no warnings 'Net::IPAM::Tree'
-use warnings::register;
-
 use Carp qw(croak);
 use Scalar::Util qw(blessed);
+###
+use namespace::clean;
 
 use Net::IPAM::Block;
 use Net::IPAM::Tree::Node;
 
+=pod
+
 =encoding utf8
 
 =head1 NAME
@@ -61,16 +62,22 @@ The complexity for tree operations is in worst case O(h * log n) with h <= 128 (
 
 =head1 METHODS
 
-=head2 new
+=head2 new([$error_cb])
 
 Create Net::IPAM::Tree object.
 
   my $t = Net::IPAM::Tree->new;
 
+The only optional argument is a coderef for an error handler.
+With no error callback L</insert> just calls C<< warn() >> on duplicate items.
+
+The error callback gets the duplicate block as argument.
+
 =cut
 
 sub new {
   my $self = bless {}, $_[0];
+
   $self->{root} = Net::IPAM::Tree::Node->new(
     {
       block  => undef,
@@ -78,6 +85,14 @@ sub new {
       childs => [],
     }
   );
+
+  if ( ref $_[1] eq 'CODE' ) {
+    $self->{error_cb} = $_[1];
+  }
+  else {
+    $self->{error_cb} = sub { warn("duplicate block during insert: $_[0]\n") };
+  }
+
   return $self;
 }
 
@@ -88,14 +103,13 @@ than inserting unsorted single blocks in a loop.
 
 Returns the tree object on success for method chaining.
 
-  my $t = Net::IPAM::Tree->new->insert(@blocks) // die("one or more blocks are duplicate");;
+  my $t = Net::IPAM::Tree->new->insert(@blocks) // die("one or more blocks are duplicate");
 
-Returns undef on duplicate blocks in the tree and generate warnings (warnings can be disabled).
+Returns undef on duplicate blocks in the tree and generate warnings.
+To shut up the warnings on duplicate items, define your own error callback in the constructor.
 
-  {
-    no warnings 'Net::IPAM::Tree';
-    $t->insert($dup)
-  }
+  my $t = Net::IPAM::Tree->new(sub{});
+  $t->insert(@blocks) // die("one or more blocks are duplicate");
 
 =cut
 
@@ -120,9 +134,7 @@ sub insert {
 
     unless ( defined $self->{root}->_insert_node($node) ) {
       $warnings++;
-
-      # caller can shut up with: no warnings 'Net::IPAM::Tree'
-      warnings::warnif("duplicate $block,");
+      $self->{error_cb}->($block);
     }
   }
 
@@ -135,8 +147,9 @@ sub insert {
 
 =head2 contains($thing)
 
-Returns true if the given $thing (L<Net::IPAM::IP> or L<Net::IPAM::Block>) is contained in any block of the tree.
-Just returns true or false and not the matching prefix, this is much faster than a full L</"lookup"> for the longest-prefix-match.
+Returns true if the given $thing (L<Net::IPAM::IP> or L<Net::IPAM::Block>) is contained
+in any block of the tree. Just returns true or false and not the matching prefix,
+this is much faster than a full L</"lookup"> for the longest-prefix-match.
 
 This can be used for fast ACL lookups.
 
